@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: CL-PPCRE; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/cl-ppcre/api.lisp,v 1.40 2004/04/20 11:37:35 edi Exp $
+;;; $Header: /usr/local/cvsrep/cl-ppcre/api.lisp,v 1.48 2004/12/06 11:34:15 edi Exp $
 
 ;;; The external API for creating and using scanners.
 
@@ -238,9 +238,9 @@ be coerced to a simple string if it isn't one already."))
            (maybe-coerce-to-simple-string target-string)
            start end))
 
-(define-compiler-macro scan (&whole form regex target-string &rest rest)
+(define-compiler-macro scan (&whole form &environment env regex target-string &rest rest)
   "Make sure that constant forms are compiled into scanners at compile time."
-  (cond ((constantp regex)
+  (cond ((constantp regex env)
           `(scan (load-time-value
                    (create-scanner ,regex))
                    ,target-string ,@rest))
@@ -276,6 +276,15 @@ structure with TARGET-STRING."
                    reg-starts
                    reg-ends)))))
 
+(define-compiler-macro scan-to-strings
+    (&whole form &environment env regex target-string &rest rest)
+  "Make sure that constant forms are compiled into scanners at compile time."
+  (cond ((constantp regex env)
+          `(scan-to-strings (load-time-value
+                              (create-scanner ,regex))
+                            ,target-string ,@rest))
+        (t form)))
+
 (defmacro register-groups-bind (var-list (regex target-string
                                                 &key start end sharedp)
                                 &body body)
@@ -299,17 +308,18 @@ share structure with TARGET-STRING."
                     `(,substr-fn (if ,sharedp
                                    #'nsubseq
                                    #'subseq))
-                    (loop for var in var-list
+                    (loop for (function var) in (normalize-var-list var-list)
                           for counter from 0
                           when var
-                          collect `(,var (let ((,start-index
-                                                 (aref ,reg-starts ,counter)))
-                                           (if ,start-index
-                                             (funcall ,substr-fn
-                                                      ,target-string
-                                                      ,start-index
-                                                      (aref ,reg-ends ,counter))
-                                             nil)))))
+                            collect `(,var (let ((,start-index
+                                                   (aref ,reg-starts ,counter)))
+                                             (if ,start-index
+                                               (funcall ,function
+                                                        (funcall ,substr-fn
+                                                                 ,target-string
+                                                                 ,start-index
+                                                                 (aref ,reg-ends ,counter)))
+                                               nil)))))
               ,@body))))))
 
 (defmacro do-scans ((match-start match-end reg-starts reg-ends regex
@@ -363,7 +373,7 @@ declarations."
                 (locally
                   ,@body)
                 ;; advance by one position if we had a zero-length match
-                (setq ,%start (if (= ,%start ,match-end)
+                (setq ,%start (if (= ,match-start ,match-end)
                                 (1+ ,match-end)
                                 ,match-end)))
               (go ,loop-tag))))))))
@@ -441,16 +451,18 @@ share structure with TARGET-STRING. BODY may start with declarations."
         (do-scans (,match-start ,match-end ,reg-starts ,reg-ends
                                 ,regex ,target-string
                                 ,result-form :start ,start :end ,end)
-          (let ,(loop for var in var-list
+          (let ,(loop for (function var) in (normalize-var-list var-list)
                       for counter from 0
-                      collect `(,var (let ((,start-index
-                                             (aref ,reg-starts ,counter)))
-                                       (if ,start-index
-                                         (funcall ,substr-fn
-                                                  ,target-string
-                                                  ,start-index
-                                                  (aref ,reg-ends ,counter))
-                                         nil))))
+                      when var
+                        collect `(,var (let ((,start-index
+                                               (aref ,reg-starts ,counter)))
+                                         (if ,start-index
+                                           (funcall ,function
+                                                    (funcall ,substr-fn
+                                                             ,target-string
+                                                             ,start-index
+                                                             (aref ,reg-ends ,counter)))
+                                           nil))))
             ,@body))))))
 
 (defun all-matches (regex target-string
@@ -474,6 +486,15 @@ the scan is continued one position behind this match."
       (push match-start result-list)
       (push match-end result-list))))
 
+(define-compiler-macro all-matches (&whole form &environment env regex &rest rest)
+   "Make sure that constant forms are compiled into scanners at
+compile time."
+   (cond ((constantp regex env)
+           `(all-matches (load-time-value
+                           (create-scanner ,regex))
+			 ,@rest))
+         (t form)))
+
 (defun all-matches-as-strings (regex target-string
                                      &key (start 0)
                                           (end (length target-string))
@@ -493,6 +514,16 @@ share structure with TARGET-STRING."
                                   :start start :end end :sharedp sharedp)
       (push match result-list))))
 
+(define-compiler-macro all-matches-as-strings (&whole form &environment env regex &rest rest)
+   "Make sure that constant forms are compiled into scanners at
+compile time."
+   (cond ((constantp regex env)
+           `(all-matches-as-strings
+	    (load-time-value
+              (create-scanner ,regex))
+	    ,@rest))
+         (t form)))
+
 (defun split (regex target-string
                     &key (start 0)
                          (end (length target-string))
@@ -569,9 +600,9 @@ TARGET-STRING."
                               target-string this-start this-end)
                      nil)))))
 
-(define-compiler-macro split (&whole form regex target-string &rest rest)
+(define-compiler-macro split (&whole form &environment env regex target-string &rest rest)
   "Make sure that constant forms are compiled into scanners at compile time."
-  (cond ((constantp regex)
+  (cond ((constantp regex env)
           `(split (load-time-value
                     (create-scanner ,regex))
                   ,target-string ,@rest))
@@ -770,7 +801,8 @@ S-expression."
                           target-string
                           start end
                           match-start match-end
-                          reg-starts reg-ends)
+                          reg-starts reg-ends
+                          simple-calls)
   (declare (optimize speed
                      (safety 0)
                      (space 0)
@@ -806,12 +838,22 @@ corresponding template."
                                    :start (svref reg-starts token)
                                    :end (svref reg-ends token))))
                  (function
-                   (write-string (funcall token
-                                          target-string
-                                          start end
-                                          match-start match-end
-                                          reg-starts reg-ends)
-                                 s))
+                   (write-string 
+                    (cond (simple-calls
+                           (apply token
+                                  (nsubseq target-string match-start match-end)
+                                  (map 'list
+                                       (lambda (reg-start reg-end)
+                                         (and reg-start
+                                              (nsubseq target-string reg-start reg-end)))
+                                       reg-starts reg-ends)))
+                          (t
+                           (funcall token
+                                    target-string
+                                    start end
+                                    match-start match-end
+                                    reg-starts reg-ends)))
+                    s))
                  (symbol
                    (case token
                      ((:backslash)
@@ -833,14 +875,25 @@ corresponding template."
                                      :start match-end
                                      :end end))
                      (otherwise
-                       (write-string (funcall token
-                                              target-string
-                                              start end
-                                              match-start match-end
-                                              reg-starts reg-ends)
-                                     s)))))))))
+                      (write-string
+                       (cond (simple-calls
+                              (apply token
+                                     (nsubseq target-string match-start match-end)
+                                     (map 'list
+                                          (lambda (reg-start reg-end)
+                                            (and reg-start
+                                                 (nsubseq target-string reg-start reg-end)))
+                                          reg-starts reg-ends)))
+                             (t
+                              (funcall token
+                                       target-string
+                                       start end
+                                       match-start match-end
+                                       reg-starts reg-ends)))
+                       s)))))))))
 
-(defun replace-aux (target-string replacement pos-list reg-list start end preserve-case)
+(defun replace-aux (target-string replacement pos-list reg-list
+                                  start end preserve-case simple-calls)
   (declare (optimize speed
                      (safety 0)
                      (space 0)
@@ -867,7 +920,8 @@ representing the corresponding register start and end positions."
                                                         target-string
                                                         start end
                                                         from to
-                                                        reg-starts reg-ends)
+                                                        reg-starts reg-ends
+                                                        simple-calls)
                                      nil)
             while to
             if replace
@@ -887,7 +941,8 @@ representing the corresponding register start and end positions."
 (defun regex-replace (regex target-string replacement
                             &key (start 0)
                             (end (length target-string))
-                            preserve-case)
+                            preserve-case
+                            simple-calls)
   (declare (optimize speed
                      (safety 0)
                      (space 0)
@@ -926,13 +981,23 @@ match."
       (replace-aux target-string replacement
                    (list match-start match-end)
                    (list reg-starts reg-ends)
-                   start end preserve-case)
+                   start end preserve-case simple-calls)
       (subseq target-string start end))))
 
+(define-compiler-macro regex-replace
+    (&whole form &environment env regex target-string replacement &rest rest)
+  "Make sure that constant forms are compiled into scanners at compile time."
+  (cond ((constantp regex env)
+          `(regex-replace (load-time-value
+                            (create-scanner ,regex))
+                          ,target-string ,replacement ,@rest))
+        (t form)))
+
 (defun regex-replace-all (regex target-string replacement
                                 &key (start 0)
                                      (end (length target-string))
-                                     preserve-case)
+                                     preserve-case
+                                     simple-calls)
   (declare (optimize speed
                      (safety 0)
                      (space 0)
@@ -978,9 +1043,18 @@ match."
       (replace-aux target-string replacement
                    (nreverse pos-list)
                    (nreverse reg-list)
-                   start end preserve-case)
+                   start end preserve-case simple-calls)
       (subseq target-string start end))))
 
+(define-compiler-macro regex-replace-all
+    (&whole form &environment env regex target-string replacement &rest rest)
+  "Make sure that constant forms are compiled into scanners at compile time."
+  (cond ((constantp regex env)
+          `(regex-replace-all (load-time-value
+                                (create-scanner ,regex))
+                              ,target-string ,replacement ,@rest))
+        (t form)))
+
 #-:cormanlisp
 (defmacro regex-apropos-aux ((regex packages case-insensitive &optional return-form)
                              &body body)
@@ -1169,3 +1243,18 @@ end-of-line comments, i.e. those starting with #\\# and ending with
                            comment-scanner)
                          string
                          #'remove-tokens))))
+
+(defun parse-tree-synonym (symbol)
+  "Returns the parse tree the SYMBOL symbol is a synonym for. Returns
+NIL is SYMBOL wasn't yet defined to be a synonym."
+  (get symbol 'parse-tree-synonym))
+
+(defun (setf parse-tree-synonym) (new-parse-tree symbol)
+  "Defines SYMBOL to be a synonm for the parse tree NEW-PARSE-TREE."
+  (setf (get symbol 'parse-tree-synonym) new-parse-tree))
+
+(defmacro define-parse-tree-synonym (name parse-tree)
+  "Defines the symbol NAME to be a synonym for the parse tree
+PARSE-TREE. Both arguments are quoted."
+  `(eval-when (:compile-toplevel :load-toplevel :execute)
+     (setf (parse-tree-synonym ',name) ',parse-tree)))
