@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: CL-PPCRE; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/cl-ppcre/closures.lisp,v 1.24 2004/04/20 11:37:35 edi Exp $
+;;; $Header: /usr/local/cvsrep/cl-ppcre/closures.lisp,v 1.25 2004/10/14 12:40:39 edi Exp $
 
 ;;; Here we create the closures which together build the final
 ;;; scanner.
@@ -571,6 +571,13 @@ against CHR-EXPR."
         (and next-pos
              (funcall next-fn next-pos))))))
 
+(defmethod create-matcher-aux ((filter filter) next-fn)
+  (let ((fn (fn filter)))
+    (lambda (start-pos)
+      (let ((next-pos (funcall fn start-pos)))
+        (and next-pos
+             (funcall next-fn next-pos))))))
+
 (defmethod create-matcher-aux ((void void) next-fn)
   ;; optimize away VOIDs: don't create a closure, just return NEXT-FN
   next-fn)
