@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: CL-PPCRE; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/cl-ppcre/optimize.lisp,v 1.19 2004/04/20 11:37:35 edi Exp $
+;;; $Header: /usr/local/cvsrep/cl-ppcre/optimize.lisp,v 1.21 2004/10/14 12:40:39 edi Exp $
 
 ;;; This file contains optimizations which can be applied to converted
 ;;; parse trees.
@@ -45,13 +45,13 @@
   (let ((total-size 0))
     (declare (type fixnum total-size))
     (dolist (string string-list)
-      (declare (type string string))
+      #-genera (declare (type string string))
       (incf total-size (length string)))
     (let ((result-string (make-sequence 'simple-string total-size))
           (curr-pos 0))
       (declare (type fixnum curr-pos))
       (dolist (string string-list)
-        (declare (type string string))
+        #-genera (declare (type string string))
         (replace result-string string :start1 curr-pos)
         (incf curr-pos (length string)))
       result-string)))
@@ -148,8 +148,8 @@ operation on REGEX."))
       regex)
     (t
       ;; otherwise (ANCHOR, BACK-REFERENCE, CHAR-CLASS, EVERYTHING,
-      ;; LOOKAHEAD, LOOKBEHIND, STR, VOID, and WORD-BOUNDARY) do
-      ;; nothing
+      ;; LOOKAHEAD, LOOKBEHIND, STR, VOID, FILTER, and WORD-BOUNDARY)
+      ;; do nothing
       regex)))
 
 (defgeneric gather-strings (regex)
@@ -310,8 +310,8 @@ operation on REGEX."))
       regex)
     (t
       ;; otherwise (ANCHOR, BACK-REFERENCE, CHAR-CLASS, EVERYTHING,
-      ;; LOOKAHEAD, LOOKBEHIND, STR, VOID, and WORD-BOUNDARY) do
-      ;; nothing
+      ;; LOOKAHEAD, LOOKBEHIND, STR, VOID, FILTER, and WORD-BOUNDARY)
+      ;; do nothing
       regex)))
 
 ;; Note that START-ANCHORED-P will be called after FLATTEN and GATHER-STRINGS.
@@ -378,6 +378,12 @@ zero-length assertion."))
       (if in-seq-p
         :zero-length
         nil))
+    (filter
+      (if (and in-seq-p
+               (len regex)
+               (zerop (len regex)))
+        :zero-length
+        nil))
     (t
       ;; BACK-REFERENCE, CHAR-CLASS, EVERYTHING, and STR
       nil)))
@@ -509,7 +515,7 @@ function called by END-STRIN.)"))
                      :case-insensitive-p :void))
     (t
       ;; (ALTERNATION, BACK-REFERENCE, BRANCH, CHAR-CLASS, EVERYTHING,
-      ;; REPETITION)
+      ;; REPETITION, FILTER)
       nil)))
 
 (defmethod end-string ((regex regex))
@@ -567,6 +573,9 @@ objects."))
 (defmethod compute-min-rest ((str str) current-min-rest)
   (+ current-min-rest (len str)))
     
+(defmethod compute-min-rest ((filter filter) current-min-rest)
+  (+ current-min-rest (or (len filter) 0)))
+    
 (defmethod compute-min-rest ((repetition repetition) current-min-rest)
   (setf (min-rest repetition) current-min-rest)
   (compute-min-rest (regex repetition) current-min-rest)
@@ -594,4 +603,4 @@ objects."))
     (t
       ;; zero min-len and no embedded regexes (ANCHOR,
       ;; BACK-REFERENCE, VOID, and WORD-BOUNDARY)
-      current-min-rest)))
\ No newline at end of file
+      current-min-rest)))
