@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: CL-PPCRE; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/cl-ppcre/scanner.lisp,v 1.21 2004/05/19 05:16:49 edi Exp $
+;;; $Header: /usr/local/cvsrep/cl-ppcre/scanner.lisp,v 1.22 2004/12/09 09:23:37 edi Exp $
 
 ;;; Here the scanner for the actual regex as well as utility scanners
 ;;; for the constant start and end strings are created.
@@ -134,7 +134,8 @@ a #\Newline."
   ;; START-POS) because we know we'll never call NEWLINE-SKIPPER on
   ;; the first iteration
   (loop for i of-type fixnum from (1- start-pos) below *end-pos*
-        thereis (and (char= (schar *string* i) #\Newline)
+        thereis (and (char= (schar *string* i)
+                            #\Newline)
                      (1+ i))))
 
 (defmacro insert-advance-fn (advance-fn)
@@ -203,6 +204,7 @@ ADVANCE-FN. This is a utility macro used by CREATE-SCANNER-AUX."
                (unless (setq *end-string-pos* (funcall end-string-test
                                                        end-test-pos))
                  (when (and (= 1 (the fixnum end-anchored-p))
+                            (> *end-pos* scan-start-pos)
                             (char= #\Newline (schar *string* (1- *end-pos*))))
                    ;; if we didn't find an end string candidate from
                    ;; END-TEST-POS and if a #\Newline at the end is
